#!/usr/bin/python3
# -*- coding: utf-8 -*-

import json
import getopt
import untangle
import os
import time
import sys
import shutil
from src.app import App
from src.utils import my_json
from sys import exit
from lxml import etree
this_dir, this_filename = os.path.split(__file__)


def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]

    inputfile = ''
    name = ''
    customer = ''
    outfolder = ''
    git_repo = ''
    opts = ''

    try:
        opts, args = getopt.getopt(argv, "hf:o:n:g:c:", ["help", "file=", "name=", "git-repository-ssh=",
                                                       "out-folder=", "customer="])
    except getopt.GetoptError:
        usage()
        exit(2)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            exit()
        elif opt in ("-f", "--file"):
            inputfile = arg
        elif opt in ("-c", "--customer-id"):
            customer = arg
        elif opt in ("-o", "--out-folder"):
            outfolder = arg
        elif opt in ("-n", "--name"):
            name = arg
        elif opt in ("-g", "--git-repository-ssh"):
            git_repo = arg

    if inputfile == '':
        print("le paramètre -f est obligatoire")
        usage()
        exit(2)
    if name == '':
        print("le paramètre -n est obligatoire")
        usage()
        exit(2)

    check_xml(inputfile)

    parse_xml(inputfile, name, customer, outfolder, git_repo)


def usage():
    print("usage :")
    print("src -f <input_xml_file> -n <app_name> [-c <customer_id>]")
    print("\t-o, --out-folder=\t\toutput folder")
    print("\t-n, --name=\t\tapplication name, permit application merge in case of name change"
          " (ex core_app")
    print("\t-f, --file=\t\t:XML file to convert")
    print("\t-g, --git-repository-ssh=\t\t: ssh url of the repository")
    print("\t-c, --customer-id=\t\tOPTIONEL : customer id, to create a new branch")


def parse_xml(xml, app_name, customer, outfolder, git_repo):
    time_start = time.perf_counter()
    print("processing {}".format(xml))

    app_tree = untangle.parse(xml)
    date_creation_fichier = os.path.getmtime(xml)
    json_path = os.path.join(this_dir, "utils", "id_dict.json")
    file_id = json.load(open(json_path))

    # objets dépendants -> ajout dans la liste de référence
    if hasattr(app_tree.Application.DependentDefs, "DataObjectDef"):
        for DataObjectDef in app_tree.Application.DependentDefs.DataObjectDef:
            obj = my_json.find_id_json(file_id, DataObjectDef["id"])
            if not obj:
                file_id[DataObjectDef["id"]] = DataObjectDef["origId"]

        jsontest = json.dumps(file_id)
        with open(json_path, "w", encoding='utf-8') as f:
            f.write(jsontest)

        del jsontest, f, file_id

    menus = ""
    hosted = ""
    props = ""
    objects = ""
    webpages = ""
    portals = ""
    seeds = ""
    description = ""
    dependents = ""
    batchs = ""

    if hasattr(app_tree.Application, "Props"):
        props = app_tree.Application.Props

    if hasattr(app_tree.Application, "DataObjectDefs"):
        if hasattr(app_tree.Application.DataObjectDefs, "DataObjectDef"):
            objects = app_tree.Application.DataObjectDefs.DataObjectDef

    if hasattr(app_tree.Application, "HostedFiles"):
        hosted = app_tree.Application.HostedFiles.HostedFile

    if hasattr(app_tree.Application, "Menus"):
        if hasattr(app_tree.Application.Menus, "Menu"):
            menus = app_tree.Application.Menus.Menu

    if hasattr(app_tree.Application, "WebPageDefs"):
        webpages = app_tree.Application.WebPageDefs.WebPageDef

    if hasattr(app_tree.Application, "Portals"):
        if hasattr(app_tree.Application.Portals, "WebSite"):
            portals = app_tree.Application.Portals.WebSite

    if hasattr(app_tree.Application, "SeedRecords"):
        seeds = app_tree.Application.SeedRecords.DataObject

    if hasattr(app_tree.Application, "Description"):
        description = app_tree.Application.Description.cdata

    if hasattr(app_tree.Application, "DependentDefs"):
        if hasattr(app_tree.Application.DependentDefs, "DataObjectDef"):
            dependents = app_tree.Application.DependentDefs.DataObjectDef

    if hasattr(app_tree.Application, "BatchJobs"):
        if hasattr(app_tree.Application.BatchJobs, "Event"):
            batchs = app_tree.Application.BatchJobs.Event

    my_app = App(
        app_name,
        app_tree.Application["id"],
        app_tree.Application["version"],
        app_tree.Application["packedId"],
        app_tree.Application.DisplayName.cdata,
        description,
        props, objects, hosted, menus, webpages, portals, seeds, dependents, batchs,
        outfolder, git_repo, date_creation_fichier
    )
    if git_repo:
        my_app.check_git_exists()
        my_app.git_set_branch(customer)

    app_dir = my_app.to_file()

    text_file = "{}/app.xml".format(app_dir.rstrip("/"))
    shutil.copy(xml, text_file)

    if git_repo:
        my_app.commit_tag_push()
        my_app.git_set_branch()
    time_elapsed = (time.perf_counter() - time_start)

    print("application '{}' generated in {}s".format(app_tree.Application.DisplayName.cdata,
                                                                       round(time_elapsed, 2)))


def check_xml(xml_to_check):
    # parse xml
    try:
        if not os.path.isfile(xml_to_check):
            raise FileNotFoundError

        with open(xml_to_check, "r", encoding='utf-8') as file:
            xml = file.read()
            xml = bytes(bytearray(xml, encoding='utf-8'))
            etree.XML(xml)

    # check for FileNotFoundError
    except FileNotFoundError:
        print("Can't find file {}".format(xml_to_check))
        exit(2)

    # check for file IO error
    except IOError as err:
        print('Invalid File')
        with open('error.log', 'w', encoding='utf-8') as error_log_file:
            error_log_file.write(str(err))

        exit(2)

    # check for XML syntax errors
    except etree.XMLSyntaxError as err:
        print("XML Syntax Error, see error.log")
        with open('error.log', 'w', encoding='utf-8') as error_log_file:
            error_log_file.write(str(err))
        exit(2)

    except Exception as err:
        print("Unknown error, see error.log")
        with open('error.log', 'w', encoding='utf-8') as error_log_file:
            error_log_file.write(str(err))
        exit(2)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import binascii
import os, re
import base64
from src.utils import my_tools


class Event(object):
    """
    Define an event
    """
    script = ""

    def __init__(self, event, dir):
        self.id = event["id"]
        self.orig_id = event["origId"]
        self.order = event["orderNo"]
        self.obj_def = event["objDef"]
        self.config_id = event["configId"]
        self.name = event["name"]
        self.on_what = event["onWhat"] # sur quel origId d'un objet
        self.relative_to_id = event["relativeToId"]
        self.delay = event["delay"]
        self.template_id = event["templateId"]
        self.change_field_id = event["changeFieldId"]
        self.id_deployed = event["isDeployed"]
        dirname = my_tools.mysplit(self.name)
        self.event_dir = "{}/{}".format(dir, dirname)
        self.properties = event.__dict__
        if hasattr(event, "ConditionB64"):
            self.script = event.ConditionB64.cdata
        if hasattr(event, "Props"):
            self.props = event.Props.__dict__

    def to_file(self):
        # properties file
        if not os.path.exists(self.event_dir):
            os.makedirs(self.event_dir)
        file = open("{}/PROPERTIES.md".format(self.event_dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDef : {}\n".format(self.obj_def))
            file.write("- configId : {}\n".format(self.id))
            file.write("- onWhat : {}\n".format(self.orig_id)) # TODO : trouver le nom dans un dict
            file.write("- relativeTo : {}\n".format(self.relative_to_id)) # TODO: idem
            file.write("- delay : {}\n".format(self.delay))
            file.write("- templateId : {}\n".format(self.template_id)) # TODO: idem
            file.write("- changeField : {}\n".format(self.change_field_id)) # TODO: idem


            # Properties
            file.write("\n\n## Properties\n")
            for el in self.properties["children"]:
                if(el.__dict__["_name"] != "ConditionB64") and (el.__dict__["_name"] != "Props"):
                    file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))

            if self.props:
                for el in self.props["children"]:
                    if el.__dict__["_name"] == "template":
                        self.template( el.__dict__["cdata"])
                    else:
                        file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))

        if self.script:
            file = open("{}/script.js".format(self.event_dir), 'w+', encoding='utf-8')
            with file:
                file_content = base64.b64decode(self.script).decode("UTF-8", "ignore")
                file.write(file_content)

    def template(self, text):
        template = open("{}/template.js".format(self.event_dir), 'w+', encoding='utf-8')
        with template:
            try:
                template_content = base64.b64decode(text).decode("UTF-8", "ignore")
                template.write(template_content)
            except binascii.Error:
                template.write(text)
            except ValueError:
                template.write(text)




# -*- coding: utf-8 -*-

import base64, os, shutil, json, datetime, unidecode, git
from git import Repo, RemoteProgress, Actor, Commit
from src.utils import my_json
from src.app_object import Object
from src.app_menu import Menu
from src.web.webpagedef import WebPageDef
from src.web.portal import Portal
from src.seedrecord import SeedRecord
from src.app_event import Event

this_dir, this_filename = os.path.split(__file__)


class App(object):
    """
    The application
    """
    header = ""
    footer = ""
    token_names = ""
    git_repo_obj = ""
    git_origin = ""
    git_branch = ""
    show_menus = ""
    use_legacy_header_footer = ""
    is_hidden = ""
    help_field = ""
    is_deployed = ""
    is_managed = ""
    pub_managed = ""
    is_logo_hidden = ""

    def __init__(self, app_name, id, version, packed_id, displayname, description, properties, objects, files,
                 menus, webpages, portals, seeds, dependents, batchs, outfolder, git_repo, creation_date):
        self.id = id
        self.app_name = app_name
        self.version = version
        self.packed_id = packed_id
        self.displayname = displayname
        self.description = description
        if hasattr(properties, "showMenus"):
            self.show_menus = properties.showMenus.cdata
        if hasattr(properties, "useLegacyHeaderFooter"):
            self.use_legacy_header_footer = properties.useLegacyHeaderFooter.cdata
        if hasattr(properties, "isHidden"):
            self.is_hidden = properties.isHidden.cdata
        if hasattr(properties, "helpField"):
            self.help_field = properties.helpField.cdata
        if hasattr(properties, "isDeployed"):
            self.is_deployed = properties.isDeployed.cdata
        if hasattr(properties, "isManaged"):
            self.is_managed = properties.isManaged.cdata
        if hasattr(properties, "header"):
            self.header = base64.b64decode(properties.header.cdata)
        if hasattr(properties, "pubManaged"):
            self.pub_managed = properties.pubManaged.cdata
        if hasattr(properties, "isLogoHidden"):
            self.is_logo_hidden = properties.isLogoHidden.cdata
        if hasattr(properties, "tokenNames"):
            self.token_names = properties.tokenNames.cdata
        self.dependent_defs = properties.dependentDefs.cdata
        if hasattr(properties, "footer"):
            self.footer = base64.b64decode(properties.footer.cdata)
        self.app_directory = "{}/{}".format(outfolder, self.app_name)
        self.objects = objects
        self.files = files
        self.menus = menus
        self.webpages = webpages
        self.portals = portals
        self.seeds = seeds
        self.dependents = dependents
        self.batchs = batchs
        self.git_repo = git_repo
        self.creation_date = creation_date
        # delete all files and folder to make git tracking easyest
        for x in ["hosted_files", "menus", "objects", "pages", "portals",
                  "seeds"]:
            if os.path.exists("{}/{}".format(self.app_directory, x)):
                shutil.rmtree("{}/{}".format(self.app_directory, x))

        for x in ["README.md", "MENUS.md", "footer.html", "header.html", "app.xml"]:
            if os.path.exists("{}/{}".format(self.app_directory, x)):
                os.remove("{}/{}".format(self.app_directory, x))
        if not os.path.exists(self.app_directory):
            os.makedirs(self.app_directory)

    def check_git_exists(self):
        if not os.path.exists(self.app_directory + "/.git"):
            # os.makedirs(self.app_directory)
            self.git_repo_obj = Repo.init(self.app_directory)
            self.git_origin = self.git_repo_obj.create_remote('origin', url=self.git_repo)
            # self.git_repo_obj\
            #     .create_head('master', self.git_origin.refs.master)\
            #     .set_tracking_branch(self.git_origin.refs.master)\
            #     .checkout()
        else:
            self.git_repo_obj = Repo(self.app_directory)
            self.git_origin = self.git_repo_obj.remote('origin')

        self.git_repo_obj.remotes.origin.push("--all")

    def git_set_branch(self, customer_id=None):
        customer_name = "master"
        if customer_id:
            file_id = json.load(open("./src/utils/customer.json"))

            for i in file_id:
                if i['id'] == int(customer_id):
                    customer_name = i["name"] \
                        .lower() \
                        .replace(' ', '_') \
                        .replace('-', '_')
                    break
            customer_name = unidecode.unidecode(customer_name)
        try:
            self.git_repo_obj.git.checkout(customer_name)
        except git.exc.GitCommandError:
            self.git_repo_obj.git.checkout('-b', customer_name)
        self.git_branch = customer_name

    def commit_tag_push(self):
        repo = self.git_repo_obj
        # add all changes
        repo.git.add(A=True)
        # tree = repo.index.write_tree()

        # create commit with xml file's date

        # Committer and Author
        cr = repo.config_reader()
        committer = Actor.committer(cr)
        author = Actor.author(cr)

        date_creation = datetime.datetime.fromtimestamp(self.creation_date)
        date_creation_git = date_creation.replace(microsecond=0).isoformat()

        # offset = altzone
        # author_time, author_offset = date_creation, offset
        # committer_time, committer_offset = date_creation, offset

        message = "Commit automatique du " + date_creation.strftime('%d/%m/%Y')

        # os.environ["GIT_AUTHOR_DATE"] = str(date_creation)
        # os.environ["GIT_COMMITTER_DATE"] = str(date_creation)

        # Do the commit thing.
        commit = repo.index.commit(message,
                                   author=author,
                                   committer=committer,
                                   commit_date=date_creation_git,
                                   author_date=date_creation_git)

        # Create TAG if version changed
        repo.remotes.origin.push(self.git_branch)
        tags = repo.tags
        if self.version not in tags:
            repo.create_tag(self.version, message='Automatic tag "{0}"'.format(self.version))
            repo.remotes.origin.push(self.version)

        # Do the push
        repo.remotes.origin.push(self.git_branch)

    def to_file(self):
        """
        Write in files application's parameters
        - README.md -> technical informations
        - header.js -> header
        - footer.js -> footer
        :return:
        """
        readme = open("{}/README.md".format(self.app_directory), 'w+', encoding='utf-8')
        with readme:
            # nom de l'src
            readme.write("# {}\n".format(self.displayname))
            readme.write("version {}\n\n".format(self.version))
            # Description
            readme.write("## Description\n")
            readme.write(self.description)
            # Informations
            readme.write("\n\n## Informations\n")
            readme.write("- id : {}\n".format(self.id))
            readme.write("- packedId : {}\n".format(self.packed_id))
            readme.write("\nInfos déploiement :\n")
            readme.write("- [{}] showMenu \n".format("X" if self.show_menus == "true" else " "))
            readme.write(
                "- [{}] useLegacyHeaderFooter\n".format("X" if self.use_legacy_header_footer == "true" else " "))
            readme.write("- [{}] isHidden\n".format("X" if self.is_hidden == "true" else " "))
            readme.write("- [{}] helpField\n".format("X" if self.help_field == "true" else " "))
            readme.write("- [{}] isDeployed\n".format("X" if self.packed_id == "true" else " "))

            # Dependants objects
            readme.write("\n## Objets dépendants:\n")
            json_path = os.path.join(this_dir, "utils", "objects_dict.json")
            objects_dict = json.load(open(json_path))
            readme.write("|Id d'origine|Id|Nom|Nom d'intégration|\n"
                         "|---|---|---|---|\n")
            # for x in self.dependent_defs.split(","):
            #     if len(x) > 0:
            #         obj = my_json.find_in_json(objects_dict, x)
            #         if obj:
            #             readme.write("|{}|{}|{}|{}|\n"
            #                          .format(obj["origId"], x, obj["name"], obj["defName"]))
            #         else:
            #             readme.write("| |{}| | |\n".format(x))

            # write in json file
            jsontest = json.dumps(objects_dict)
            with open(json_path, "w", encoding='utf-8') as f:
                f.write(jsontest)

            del jsontest

            if self.dependents:
                for dependent in self.dependents:
                    obj = my_json.find_in_json(objects_dict, dependent["id"])
                    readme.write("|{}|{}|{}|{}|\n"
                                 .format(dependent["origId"], dependent["id"], dependent.SingularName.cdata,
                                         dependent["objDefName"]))

            # Champs de fusion
            readme.write("\n## Champs de fusion :\n")
            for x in self.token_names.split("|"):
                if len(x) > 0:
                    readme.write("- {}\n".format(x))

        if self.header:
            header = open("{}/header.html".format(self.app_directory), 'w+', encoding='utf-8')
            with header:
                header.write(self.header.decode("utf-8"))

        if self.footer:
            footer = open("{}/footer.html".format(self.app_directory), 'w+', encoding='utf-8')
            with footer:
                footer.write(self.footer.decode("utf-8"))

        self.create_hosted_files()

        self.create_objects()

        if self.menus:
            self.create_menus()

        if self.webpages:
            self.create_web_pages()

        if self.portals:
            self.create_portals()

        if self.seeds:
            self.create_seeds()

        if self.batchs:
            self.create_batchs()

        return self.app_directory

    def create_objects(self):
        # objets de l'application
        for appObject in self.objects:
            new_object = Object(appObject, self.app_directory)
            new_object.create()

    def create_menus(self):
        # Create a menu folder + menus
        # create a file MENUS.md with tree menu
        path = "{}/menus".format(self.app_directory)
        if not os.path.exists(path):
            os.makedirs(path)
        menu_file = open("{}/MENUS.md".format(self.app_directory), "w+", encoding='utf-8')
        menu_file.write("## Menus\n\n```\n.\n")
        for menu in self.menus:
            new_menu = Menu(menu, path)
            new_menu.to_file()
            menu_file.write(new_menu.get_menus(self.menus))
        menu_file.write("```\n")

    def create_hosted_files(self):
        ### HOSTED FILES
        hosted_files_dir = "{}/hosted_files".format(self.app_directory)
        for file in self.files:
            if not os.path.exists(hosted_files_dir):
                os.mkdir(hosted_files_dir)
                os.mkdir("{}/css".format(hosted_files_dir))
                os.mkdir("{}/js".format(hosted_files_dir))
                os.mkdir("{}/images".format(hosted_files_dir))

            folder = ""
            ext = ""
            is_image = False

            if file["contentType"] == "text/css":
                folder = ext = "css"
            elif file["contentType"] == "application/javascript":
                folder = ext = "js"
            elif file["contentType"] == "application/x-javascript":
                folder = "js"
                ext = "min.js"
            elif file["contentType"] == "image/png":
                folder = "images"
                ext = "png"
                is_image = True
            elif file["contentType"] == "image/jpeg":
                folder = "images"
                ext = "jpg"
                is_image = True

            if folder and not is_image:
                with open("{}/{}/#HOSTED_FILE.{}-{}.{}".format(hosted_files_dir, folder, file["origId"], file.DisplayName.cdata
                        .replace("/", "-")
                        .replace(" ", "_")
                        .replace("\\", "-")
                        .replace("&", "")
                        .replace("!", "")
                        , ext), 'w+', encoding='utf-8') as readme:
                    file_content = base64.b64decode(file.RawData.cdata).decode("iso8859_15", "ignore")
                    readme.write(file_content)
            elif folder and is_image:
                with open("{}/{}/#HOSTED_FILE.{}-{}.{}".format(hosted_files_dir, folder,file["origId"], file.DisplayName.cdata
                        .replace("/", "-")
                        .replace(" ", "_")
                        .replace("\\", "-")
                        .replace("&", "")
                        .replace("!", "")
                        , ext), 'wb') as readme:
                    if hasattr(file, "RawData"):
                        file_content = base64.b64decode(file.RawData.cdata)
                        readme.write(file_content)

    def create_web_pages(self):
        # pages
        for webpage in self.webpages:
            new_webpagedef = WebPageDef(webpage, self.app_directory)
            new_webpagedef.to_file()

    def create_portals(self):
        # portals
        for portal in self.portals:
            new_portal = Portal(portal, self.app_directory)
            new_portal.to_file()

    def create_seeds(self):
        # seed records
        for seed in self.seeds:
            new_seed = SeedRecord(seed, self.app_directory)
            new_seed.to_file()

    def create_batchs(self):
        # jobs
        for batch in self.batchs:
            event = Event(batch, self.app_directory + "/jobs/")
            event.to_file()


class MyProgressPrinter(RemoteProgress):
    def update(self, op_code, cur_count, max_count=None, message=''):
        print(op_code, cur_count, max_count, cur_count / (max_count or 100.0), message or "NO MESSAGE")
